"use strict";
var ts = require("typescript");
var path = require("path");
var errors_1 = require("./../errors");
var utils_1 = require("./../utils");
var TsTypeChecker_1 = require("./utils/TsTypeChecker");
var TsCache_1 = require("./utils/TsCache");
var TsSourceFile_1 = require("./TsSourceFile");
var TsMain = (function () {
    function TsMain(compilerOptionsResolver, fileNames, options) {
        this.tsCache = new TsCache_1.TsCache();
        this.verifyFilesExist(fileNames);
        var compilerOptions = compilerOptionsResolver.getCompilerOptions(options);
        this.program = ts.createProgram(fileNames, compilerOptions, options.compilerHost);
        this.typeChecker = this.program.getTypeChecker();
    }
    TsMain.prototype.getSourceFiles = function () {
        var _this = this;
        var tsTypeChecker = new TsTypeChecker_1.TsTypeChecker(this.typeChecker);
        return this.program.getSourceFiles().filter(function (file) {
            var baseName = path.basename(file.fileName);
            return baseName !== "lib.d.ts" && baseName !== "lib.es6.d.ts";
        }).map(function (sourceFile) {
            return new TsSourceFile_1.TsSourceFile({
                tsCache: _this.tsCache,
                sourceFile: sourceFile,
                symbol: tsTypeChecker.getSymbolAtLocation(sourceFile),
                typeChecker: tsTypeChecker
            });
        });
    };
    TsMain.prototype.getTypeScriptTypeChecker = function () {
        return this.typeChecker;
    };
    TsMain.prototype.verifyFilesExist = function (filePaths) {
        var _this = this;
        filePaths.forEach(function (filePath) { return _this.verifyFileExists(filePath); });
    };
    TsMain.prototype.verifyFileExists = function (filePath) {
        // unfortunately the ts compiler doesn't do things asynchronously so for now we won't either
        if (!utils_1.FileUtils.fileExistsSync(filePath)) {
            throw new errors_1.FileNotFoundError(utils_1.FileUtils.getAbsolutePath(filePath));
        }
    };
    return TsMain;
}());
exports.TsMain = TsMain;

//# sourceMappingURL=TsMain.js.map
