"use strict";
var __extends = (this && this.__extends)/* istanbul ignore next */ || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var ts = require("typescript");
var utils_1 = require("./../utils");
var TsSourceFileChild_1 = require("./TsSourceFileChild");
var TsSymbol_1 = require("./TsSymbol");
var TsSignature_1 = require("./TsSignature");
var TypeToStringWriter_1 = require("./TypeToStringWriter");
var TsType = (function (_super) {
    __extends(TsType, _super);
    function TsType(opts) {
        var _this = _super.call(this, opts) || this;
        _this.typeToStringWriter = new TypeToStringWriter_1.TypeToStringWriter();
        _this.type = opts.type;
        _this.node = opts.node;
        return _this;
    }
    TsType.prototype.getText = function () {
        var text = "";
        try {
            text = this.typeToStringWriter.getString(this);
        }
        catch (ex) {
            utils_1.Logger.warn(ex);
        }
        return text || this.getTypeCheckerTypeText();
    };
    TsType.prototype.getTypeCheckerTypeText = function () {
        var formatFlags = (ts.TypeFormatFlags.UseTypeOfFunction | ts.TypeFormatFlags.NoTruncation | ts.TypeFormatFlags.UseFullyQualifiedType |
            ts.TypeFormatFlags.WriteTypeArgumentsOfSignature);
        if (this.isMainTypeAliasType()) {
            formatFlags |= ts.TypeFormatFlags.InTypeAlias;
        }
        return this.typeChecker.typeToString(this.type, this.node, formatFlags) || "";
    };
    TsType.prototype.getLiteralText = function () {
        return this.type.text || "";
    };
    TsType.prototype.getProperties = function () {
        var _this = this;
        var properties = this.type.getProperties();
        return (properties || []).filter(function (p) { return p.name !== "prototype"; }).map(function (property) { return _this.createSymbol(property); });
    };
    TsType.prototype.getCallSignatures = function () {
        var _this = this;
        var callSignatures = this.type.getCallSignatures();
        return (callSignatures || []).map(function (callSignature) { return _this.createTsSignature({ signature: callSignature }); });
    };
    TsType.prototype.getTypeArguments = function () {
        var _this = this;
        var tsTypeArguments = this.type.typeArguments || [];
        return tsTypeArguments.map(function (arg) { return utils_1.tryGet(_this.getTypeCheckerTypeText(), function () { return _this.createType(arg); }); }).filter(function (arg) { return arg != null; });
    };
    TsType.prototype.getAliasTypeArguments = function () {
        var _this = this;
        var tsTypeArguments = this.type.aliasTypeArguments || [];
        return tsTypeArguments.map(function (arg) { return utils_1.tryGet(_this.getTypeCheckerTypeText(), function () { return _this.createType(arg); }); }).filter(function (arg) { return arg != null; });
    };
    TsType.prototype.getAliasSymbol = function () {
        return this.type.aliasSymbol == null ? null : this.createSymbol(this.type.aliasSymbol);
    };
    TsType.prototype.getSymbol = function () {
        return this.type.symbol == null ? null : this.createSymbol(this.type.symbol);
    };
    TsType.prototype.getSymbols = function () {
        var _this = this;
        var typeArray = this.type.types;
        if (this.type.aliasSymbol != null) {
            return [this.createSymbol(this.type.aliasSymbol)];
        }
        else if (!this.isEnumType() && typeArray != null) {
            return typeArray.map(function (t) { return t.symbol; }).filter(function (s) { return s != null; }).map(function (s) { return _this.createSymbol(s); });
        }
        else if (this.type.symbol != null) {
            return [this.createSymbol(this.type.symbol)];
        }
        else {
            return [];
        }
    };
    TsType.prototype.getUnionOrIntersectionTypes = function () {
        var _this = this;
        return (this.type.types || []).map(function (t) { return _this.createType(t); });
    };
    TsType.prototype.getArrayElementType = function () {
        var type = this.type.elementType || this.getArrayTypeArgument();
        return type == null ? null : this.createType(type);
    };
    TsType.prototype.isArrayType = function () {
        return this.getArrayElementType() != null;
    };
    TsType.prototype.isBooleanType = function () {
        return (this.type.flags & ts.TypeFlags.Boolean) !== 0;
    };
    TsType.prototype.isEnumType = function () {
        return (this.type.flags & ts.TypeFlags.Enum) !== 0;
    };
    TsType.prototype.isTupleType = function () {
        return (this.getObjectFlags() & ts.ObjectFlags.Tuple) !== 0;
    };
    TsType.prototype.isIntersectionType = function () {
        return (this.type.flags & ts.TypeFlags.Intersection) !== 0 && !this.isEnumType() && !this.isBooleanType();
    };
    TsType.prototype.isUnionType = function () {
        return (this.type.flags & ts.TypeFlags.Union) !== 0 && !this.isEnumType() && !this.isBooleanType();
    };
    TsType.prototype.isAnonymousFunctionType = function () {
        var symbol = this.getSymbol();
        if (!this.isAnonymousType() || symbol == null)
            return false;
        var node = symbol.getNodes()[0];
        if (node == null)
            return false;
        return node.isFunctionType();
    };
    TsType.prototype.isAnonymousType = function () {
        return (this.getObjectFlags() & ts.ObjectFlags.Anonymous) !== 0;
    };
    TsType.prototype.isReferenceType = function () {
        return (this.getObjectFlags() & ts.ObjectFlags.Reference) !== 0;
    };
    TsType.prototype.isClassType = function () {
        return (this.getObjectFlags() & ts.ObjectFlags.Class) !== 0;
    };
    TsType.prototype.isInterfaceType = function () {
        return (this.getObjectFlags() & ts.ObjectFlags.Interface) !== 0;
    };
    TsType.prototype.isMainTypeAliasType = function () {
        return this.node != null && this.node.kind === ts.SyntaxKind.TypeAliasDeclaration && this.typeChecker.getTypeAtLocation(this.node) === this.type;
    };
    TsType.prototype.getObjectFlags = function () {
        return this.type.objectFlags || 0;
    };
    TsType.prototype.getArrayTypeArgument = function () {
        var typeRef = this.type;
        if (typeRef.typeArguments != null && typeRef.typeArguments.length === 1 &&
            this.type.symbol != null && (this.type.symbol || {}).name === "Array") {
            return typeRef.typeArguments[0];
        }
        else {
            return null;
        }
    };
    TsType.prototype.createType = function (type) {
        return new TsType({
            sourceFile: this.sourceFile,
            tsSourceFile: this.tsSourceFile,
            typeChecker: this.typeChecker,
            tsCache: this.tsCache,
            type: type,
            node: this.node
        });
    };
    TsType.prototype.createSymbol = function (symbol) {
        var _this = this;
        return this.tsCache.getSymbol(symbol, function () {
            return new TsSymbol_1.TsSymbol({
                sourceFile: _this.sourceFile,
                typeChecker: _this.typeChecker,
                tsCache: _this.tsCache,
                symbol: symbol,
                tsSourceFile: _this.tsSourceFile
            });
        });
    };
    TsType.prototype.createTsSignature = function (opts) {
        return new TsSignature_1.TsSignature({
            sourceFile: this.sourceFile,
            typeChecker: this.typeChecker,
            tsCache: this.tsCache,
            signature: opts.signature,
            node: this.node,
            tsSourceFile: this.tsSourceFile
        });
    };
    return TsType;
}(TsSourceFileChild_1.TsSourceFileChild));
__decorate([
    utils_1.Memoize
], TsType.prototype, "getText", null);
__decorate([
    utils_1.Memoize
], TsType.prototype, "getTypeCheckerTypeText", null);
exports.TsType = TsType;

//# sourceMappingURL=TsType.js.map
