"use strict";
var FormatFlags;
(function (FormatFlags) {
    FormatFlags[FormatFlags["None"] = 0] = "None";
    FormatFlags[FormatFlags["ParentIntersection"] = 1] = "ParentIntersection";
    FormatFlags[FormatFlags["ParentUnion"] = 2] = "ParentUnion";
    FormatFlags[FormatFlags["InUnionOrIntersection"] = 4] = "InUnionOrIntersection";
    FormatFlags[FormatFlags["InTypeAlias"] = 8] = "InTypeAlias";
})(/* istanbul ignore next */FormatFlags || (FormatFlags = {}));
var TypeToStringWriter = (function () {
    /* istanbul ignore next */ function TypeToStringWriter() {
    }
    TypeToStringWriter.prototype.getString = function (type) {
        var formatFlags = FormatFlags.None;
        if (type.isMainTypeAliasType())
            formatFlags |= FormatFlags.InTypeAlias;
        return this.getStringInternal(type, formatFlags);
    };
    TypeToStringWriter.prototype.getStringInternal = function (type, formatFlags) {
        if ((formatFlags & FormatFlags.InTypeAlias) === 0) {
            var aliasSymbol = type.getAliasSymbol();
            if (aliasSymbol != null)
                return this.getTypeFromSymbol(aliasSymbol, type.getAliasTypeArguments());
        }
        else
            formatFlags &= ~FormatFlags.InTypeAlias;
        if (type.isUnionType())
            return this.getUnionType(type, formatFlags);
        else if (type.isIntersectionType())
            return this.getIntersectionType(type, formatFlags);
        var symbol = type.getSymbol();
        if (type.isAnonymousFunctionType()) {
            if (formatFlags & FormatFlags.InUnionOrIntersection)
                return "(" + type.getTypeCheckerTypeText() + ")";
            return type.getTypeCheckerTypeText();
        }
        else if (type.isAnonymousType() && symbol == null)
            return this.getTypeFromObject(type);
        else
            return type.getTypeCheckerTypeText();
    };
    TypeToStringWriter.prototype.getTypeFromObject = function (type) {
        var _this = this;
        var props = type.getProperties();
        var str = "{ ";
        props.forEach(function (prop) {
            if (prop.isPropertyReadonly())
                str += "readonly ";
            str += prop.getName();
            if (prop.isPropertyOptional())
                str += "?";
            str += ": ";
            str += _this.getStringInternal(prop.getType(), FormatFlags.None);
            str += "; ";
        });
        str += "}";
        return str;
    };
    TypeToStringWriter.prototype.getTypeFromSymbol = function (symbol, typeArguments) {
        var _this = this;
        var parentSymbol = symbol.getParentSymbol();
        var name = symbol.getName();
        if (parentSymbol != null && !parentSymbol.isFileSymbol())
            name = this.getTypeFromSymbol(parentSymbol, []) + "." + name;
        if (typeArguments.length === 0)
            return name;
        return name + "<" + typeArguments.map(function (a) { return _this.getStringInternal(a, FormatFlags.None); }).join(", ") + ">";
    };
    TypeToStringWriter.prototype.getUnionType = function (type, flags) {
        var _this = this;
        var childFlags = (flags & ~FormatFlags.ParentIntersection) | FormatFlags.ParentUnion | FormatFlags.InUnionOrIntersection;
        var unionTypes = type.getUnionOrIntersectionTypes();
        var str = unionTypes.map(function (t) { return _this.getStringInternal(t, childFlags); }).join(" | ");
        if (flags & FormatFlags.ParentIntersection)
            return "(" + str + ")";
        return str;
    };
    TypeToStringWriter.prototype.getIntersectionType = function (type, flags) {
        var _this = this;
        var childFlags = (flags & ~FormatFlags.ParentUnion) | FormatFlags.ParentIntersection | FormatFlags.InUnionOrIntersection;
        var str = type.getUnionOrIntersectionTypes().map(function (t) { return _this.getStringInternal(t, childFlags); }).join(" & ");
        if (flags & FormatFlags.ParentUnion)
            return "(" + str + ")";
        return str;
    };
    return TypeToStringWriter;
}());
exports.TypeToStringWriter = TypeToStringWriter;

//# sourceMappingURL=TypeToStringWriter.js.map
