"use strict";
var assert = require("assert");
var main_1 = require("./../../../main");
var code = "/**\n * Some class description\n */\n@ClassDecorator\n@ClassDecorator2(1, 2, \"3\")\nabstract class MyClass {\n    /**\n     * Some static property description\n     */\n    @StaticPropertyDecorator\n    @StaticPropertyDecorator2()\n    static readonly myStaticString: string;\n\n    /**\n     * Some static method description\n     */\n    @StaticMethodDecorator\n    @StaticMethodDecorator2()\n    static myStaticMethod() {\n    }\n\n    static async myAsyncStaticMethod() {\n    }\n\n    protected myString: string;\n    private myPrivateString: string;\n    /**\n     * Some property description\n     */\n    @PropertyDecorator\n    @PropertyDecorator2()\n    nonOptionalString = \"text\";\n    optionalNumber?: number;\n    protected abstract readonly myAbstractReadonlyProperty: string;\n    protected abstract get myAbstractAccessorProperty(): string;\n    protected abstract set myAbstractAccessorProperty(value: string);\n\n    get myGet(): string {\n        return \"\";\n    }\n\n    set mySet(value: string) {\n    }\n\n    get myGetSet(): string {\n        return \"\";\n    }\n\n    set myGetSet(value: string) {\n    }\n\n    private get myGetWithWriteDefined(): string {\n        return \"\";\n    }\n\n    private set mySetWithWriteDefined(value: string) {\n    }\n\n    protected get myGetSetWithWriteDefined(): string {\n        return \"\";\n    }\n\n    protected set myGetSetWithWriteDefined(value: string) {\n    }\n\n    /**\n     * Some constructure description\n     */\n    constructor(@ParameterDecorator @ParameterDecorator2(\"text\") myParam: string, public readonly myPublicParam: any, protected myProtectedParam: any, private myPrivateParam: any) {\n    }\n\n    abstract myAbstractMethod(): string;\n    abstract myAbstractMethod2(): string;\n    abstract myAbstractMethod2(str?: string): string;\n\n    /**\n     * Some method description\n     */\n    @MethodDecorator\n    @MethodDecorator2()\n    myMethod() {\n    }\n\n    myParamTypeGuard(@ParameterDecorator @ParameterDecorator2() p: string | MyClass): p is MyClass {\n    }\n\n    myThisTypeGuard(): this is MyClass {\n    }\n\n    private myMethodWithOverloadSignatures<T>(): string;\n    private myMethodWithOverloadSignatures<T>(str?: string) {\n        return \"\";\n    }\n\n    private myPrivateMethod() {\n    }\n\n    private *myGeneratorMethod() {\n    }\n\n    async myAsyncMethod() {\n    }\n\n    abstract async myAbstractAsyncMethod();\n}\n\nclass MyClassWithPrivateConstructor {\n    private constructor() {\n    }\n}\n\nclass MyTypeParameterClass<T> {\n}\n\nclass MyChildClass extends MyTypeParameterClass<string> {\n}\n\nclass MyImplementsClass implements MyChildClass {\n}\n\nclass MyExtendsImplementsClass extends MyChildClass implements MyImplementsClass {\n}\n\nclass MyExtendsArray extends Array<string> {\n}\n\nclass MyClassWithOverloadSignatures {\n    constructor(param: number);\n    constructor(param: string);\n    constructor(param: any) {\n    }\n}\n";
describe("ClassDefinition", function () {
    var file = main_1.getInfoFromString(code);
    describe("write()", function () {
        describe("MyClass", function () {
            it("should contain the class written out", function () {
                var expected = "/**\n * Some class description\n */\n@ClassDecorator\n@ClassDecorator2(1, 2, \"3\")\nabstract class MyClass {\n    /**\n     * Some static property description\n     */\n    @StaticPropertyDecorator\n    @StaticPropertyDecorator2()\n    static readonly myStaticString: string;\n\n    /**\n     * Some static method description\n     */\n    @StaticMethodDecorator\n    @StaticMethodDecorator2()\n    static myStaticMethod() {\n    }\n\n    static async myAsyncStaticMethod() {\n    }\n\n    protected myString: string;\n    private myPrivateString: string;\n    /**\n     * Some property description\n     */\n    @PropertyDecorator\n    @PropertyDecorator2()\n    nonOptionalString = \"text\";\n    optionalNumber?: number;\n    protected abstract readonly myAbstractReadonlyProperty: string;\n    protected abstract get myAbstractAccessorProperty(): string;\n    protected abstract set myAbstractAccessorProperty(value: string);\n    myGet: string;\n    mySet: string;\n    myGetSet: string;\n\n    private get myGetWithWriteDefined(): string {\n        return \"\";\n    }\n\n    private set mySetWithWriteDefined(value: string) {\n        alert(value);\n    }\n\n    protected get myGetSetWithWriteDefined(): string {\n        return \"\";\n    }\n\n    protected set myGetSetWithWriteDefined(value: string) {\n        alert(value);\n    }\n\n    /**\n     * Some constructure description\n     */\n    constructor(@ParameterDecorator @ParameterDecorator2(\"text\") myParam: string, public readonly myPublicParam: any, protected myProtectedParam: any, private myPrivateParam: any) {\n    }\n\n    abstract myAbstractMethod(): string;\n    abstract myAbstractMethod2(): string;\n    abstract myAbstractMethod2(str?: string): string;\n\n    /**\n     * Some method description\n     */\n    @MethodDecorator\n    @MethodDecorator2()\n    myMethod() {\n    }\n\n    myParamTypeGuard(@ParameterDecorator @ParameterDecorator2() p: string | MyClass): p is MyClass {\n    }\n\n    myThisTypeGuard(): this is MyClass {\n    }\n\n    private myMethodWithOverloadSignatures<T>(): string;\n    private myMethodWithOverloadSignatures<T>(str?: string): string {\n    }\n\n    private myPrivateMethod() {\n    }\n\n    private *myGeneratorMethod() {\n    }\n\n    async myAsyncMethod() {\n    }\n\n    abstract async myAbstractAsyncMethod(): any;\n}\n";
                var classDef = file.classes[0];
                var getSetProperty = classDef.getProperty("myGetSetWithWriteDefined");
                getSetProperty.onWriteGetBody = function (writer) { return writer.write("return \"\";"); };
                getSetProperty.onWriteSetBody = function (writer) { return writer.write("alert(value);"); };
                classDef.getProperty("myGetWithWriteDefined").onWriteGetBody = function (writer) { return writer.write("return \"\";"); };
                classDef.getProperty("mySetWithWriteDefined").onWriteSetBody = function (writer) { return writer.write("alert(value);"); };
                assert.equal(classDef.write(), expected);
            });
        });
        describe("MyClassWithPrivateConstructor", function () {
            it("should contain the class written out", function () {
                var expected = "class MyClassWithPrivateConstructor {\n    private constructor() {\n    }\n}\n";
                assert.equal(file.classes[1].write(), expected);
            });
        });
        describe("MyTypeParameterClass", function () {
            it("should contain the class written out", function () {
                var expected = "class MyTypeParameterClass<T> {\n}\n";
                assert.equal(file.classes[2].write(), expected);
            });
        });
        describe("MyChildClass", function () {
            it("should contain the class written out", function () {
                var expected = "class MyChildClass extends MyTypeParameterClass<string> {\n}\n";
                assert.equal(file.classes[3].write(), expected);
            });
        });
        describe("MyImplementsClass", function () {
            it("should contain the class written out", function () {
                var expected = "class MyImplementsClass implements MyChildClass {\n}\n";
                assert.equal(file.classes[4].write(), expected);
            });
        });
        describe("MyExtendsImplementsClass", function () {
            it("should contain the class written out", function () {
                var expected = "class MyExtendsImplementsClass extends MyChildClass implements MyImplementsClass {\n}\n";
                assert.equal(file.classes[5].write(), expected);
            });
        });
        describe("MyExtendsArray", function () {
            it("should contain the class written out", function () {
                var expected = "class MyExtendsArray extends Array<string> {\n}\n";
                assert.equal(file.classes[6].write(), expected);
            });
        });
        describe("MyClassWithOverloadSignatures", function () {
            it("should contain the class written out", function () {
                var expected = "class MyClassWithOverloadSignatures {\n    constructor(param: number);\n    constructor(param: string);\n    constructor(param: any) {\n    }\n}\n";
                assert.equal(file.classes[7].write(), expected);
            });
        });
    });
});

//# sourceMappingURL=classWriteTests.js.map
